@@ -31,6 +31,18 @@
 
 #include <config.h>
 
+#ifdef ENABLE_NLS
+# define _(string) gettext(string)
+# include "gettext.h"
+# include <locale.h>
+# include <langinfo.h>
+# include <iconv.h>
+#else
+# define _(string) string
+#endif
+
+# define N_(string) string
+
 #include "libdvbv5/dvb-file.h"
 #include "libdvbv5/dvb-demux.h"
 #include "libdvbv5/dvb-scan.h"
@@ -44,9 +56,12 @@ struct arguments {
 };
 
 static const struct argp_option options[] = {
-	{"input-format",	'I',	"format",	0, "Valid input formats: ZAP, CHANNEL, DVBV5", 0},
-	{"output-format",	'O',	"format",	0, "Valid output formats: VDR, ZAP, CHANNEL, DVBV5", 0},
-	{"delsys",		's',	"system",	0, "Delivery system type. Needed if input or output format is ZAP", 0},
+	{"input-format",	'I',	N_("format"),	0, N_("Valid input formats: ZAP, CHANNEL, DVBV5"), 0},
+	{"output-format",	'O',	N_("format"),	0, N_("Valid output formats: VDR, ZAP, CHANNEL, DVBV5"), 0},
+	{"delsys",		's',	N_("system"),	0, N_("Delivery system type. Needed if input or output format is ZAP"), 0},
+	{"help",        '?',	0,		0,	N_("Give this help list"), -1},
+	{"usage",	-3,	0,		0,	N_("Give a short usage message")},
+	{"version",	'V',	0,		0,	N_("Print program version"), -1},
 	{ 0, 0, 0, 0, 0, 0 }
 };
 
@@ -66,6 +81,18 @@ static error_t parse_opt(int k, char *optarg, struct argp_state *state)
 	case 's':
 		args->delsys = dvb_parse_delsys(optarg);
 		break;
+	case '?':
+		argp_state_help(state, state->out_stream,
+				ARGP_HELP_SHORT_USAGE | ARGP_HELP_LONG
+				| ARGP_HELP_DOC);
+		fprintf(state->out_stream, _("\nReport bugs to %s.\n"), argp_program_bug_address);
+		exit(0);
+	case 'V':
+		fprintf (state->out_stream, "%s\n", argp_program_version);
+		exit(0);
+	case -3:
+		argp_state_help(state, state->out_stream, ARGP_HELP_USAGE);
+		exit(0);
 	default:
 		return ARGP_ERR_UNKNOWN;
 	};
@@ -77,16 +104,16 @@ static int convert_file(struct arguments *args)
 	struct dvb_file *dvb_file = NULL;
 	int ret;
 
-	printf("Reading file %s\n", args->input_file);
+	printf(_("Reading file %s\n"), args->input_file);
 
 	dvb_file = dvb_read_file_format(args->input_file, args->delsys,
 				    args->input_format);
 	if (!dvb_file) {
-		fprintf(stderr, "Error reading file %s\n", args->input_file);
+		fprintf(stderr, _("Error reading file %s\n"), args->input_file);
 		return -1;
 	}
 
-	printf("Writing file %s\n", args->output_file);
+	printf(_("Writing file %s\n"), args->output_file);
 	ret = dvb_write_file_format(args->output_file, dvb_file,
 				    args->delsys, args->output_format);
 	dvb_file_free(dvb_file);
@@ -101,12 +128,16 @@ int main(int argc, char **argv)
 	const struct argp argp = {
 		.options = options,
 		.parser = parse_opt,
-		.doc = "scan DVB services using the channel file",
-		.args_doc = "<input file> <output file>",
+		.doc = N_("scan DVB services using the channel file"),
+		.args_doc = N_("<input file> <output file>"),
 	};
 
+	setlocale (LC_ALL, "");
+	bindtextdomain (PACKAGE, LOCALEDIR);
+	textdomain (PACKAGE);
+
 	memset(&args, 0, sizeof(args));
-	argp_parse(&argp, argc, argv, 0, &idx, &args);
+	argp_parse(&argp, argc, argv, ARGP_NO_HELP | ARGP_NO_EXIT, &idx, &args);
 
 	if (idx + 1 < argc) {
 		args.input_file = argv[idx];
@@ -114,20 +145,21 @@ int main(int argc, char **argv)
 	}
 
 	if (args.input_format == FILE_UNKNOWN) {
-		fprintf(stderr, "ERROR: Please specify a valid input format\n");
+		fprintf(stderr, _("ERROR: Please specify a valid input format\n"));
 		missing = 1;
 	} else if (!args.input_file) {
-		fprintf(stderr, "ERROR: Please specify a valid input file\n");
+		fprintf(stderr, _("ERROR: Please specify a valid input file\n"));
 		missing = 1;
 	} else if (args.output_format == FILE_UNKNOWN) {
-		fprintf(stderr, "ERROR: Please specify a valid output format\n");
+		fprintf(stderr, _("ERROR: Please specify a valid output format\n"));
 		missing = 1;
 	} else if (!args.output_file) {
-		fprintf(stderr, "ERROR: Please specify a valid output file\n");
+		fprintf(stderr, _("ERROR: Please specify a valid output file\n"));
 		missing = 1;
 	} else if (((args.input_format == FILE_ZAP) ||
-		   (args.output_format == FILE_ZAP)) && args.delsys <= 0) {
-		fprintf(stderr, "ERROR: Please specify a valid delivery system for ZAP format\n");
+		   (args.output_format == FILE_ZAP)) &&
+		   (args.delsys <= 0 || args.delsys == SYS_ISDBS)) {
+		fprintf(stderr, _("ERROR: Please specify a valid delivery system for ZAP format\n"));
 		missing = 1;
 	}
 	if (missing) {
