@@ -114,7 +114,6 @@ private:
 	void stopAudio();
 	bool startOutput();
 	void stopOutput();
-	__u32 defaultColorspace(bool capture);
 
 	bool m_clear[64];
 	cv4l_fmt m_capSrcFormat;
@@ -125,10 +124,16 @@ private:
 	bool m_mustConvert;
 	CapMethod m_capMethod;
 	bool m_makeSnapshot;
+	bool m_singleStep;
 	RenderMethod m_renderMethod;
+	int m_overrideColorspace;
+	int m_overrideXferFunc;
+	int m_overrideYCbCrEnc;
+	int m_overrideQuantization;
 
 private slots:
 	void capStart(bool);
+	void capStep(bool);
 	void outStart(bool);
 	void makeFullScreen(bool);
 	void capFrame();
@@ -155,10 +160,12 @@ private slots:
 	void enableScaling(bool enable);
 	void updatePixelAspectRatio();
 	void updateCropping();
-	void updateColorspace();
-	void updateDisplayColorspace();
 	void clearBuffers();
 	void about();
+	void overrideColorspaceChanged(QAction *a);
+	void overrideXferFuncChanged(QAction *a);
+	void overrideYCbCrEncChanged(QAction *a);
+	void overrideQuantChanged(QAction *a);
 
 	// tpg
 private slots:
@@ -170,11 +177,12 @@ private slots:
 	void insSAVChanged(int val);
 	void insEAVChanged(int val);
 	void videoAspectRatioChanged(int val);
-	void colorspaceChanged(int val);
 	void limRGBRangeChanged(int val);
 	void fillPercentageChanged(int val);
 	void alphaComponentChanged(int val);
 	void applyToRedChanged(int val);
+	__u32 tpgDefaultColorspace();
+	void tpgColorspaceChanged();
 
 public:
 	virtual void error(const QString &text);
@@ -185,11 +193,16 @@ public:
 	void info(const QString &info);
 	virtual void closeEvent(QCloseEvent *event);
 	void updateLimRGBRange();
+	void updateColorspace();
 	QAction *m_resetScalingAct;
 	QAction *m_useBlendingAct;
 	QAction *m_useLinearAct;
 	QAction *m_snapshotAct;
 	QAction *m_showFramesAct;
+	QMenu *m_overrideColorspaceMenu;
+	QMenu *m_overrideXferFuncMenu;
+	QMenu *m_overrideYCbCrEncMenu;
+	QMenu *m_overrideQuantizationMenu;
 
 private:
 	void addWidget(QGridLayout *grid, QWidget *w, Qt::Alignment align = Qt::AlignLeft);
@@ -227,11 +240,16 @@ private:
 	void updateFreqChannel();
 	bool showFrames();
 
+	// tpg
 	struct tpg_data m_tpg;
 	v4l2_std_id m_tpgStd;
 	unsigned m_tpgField;
+	bool m_tpgFieldAlt;
 	unsigned m_tpgSizeImage;
-	unsigned m_tpgColorspace;
+	QComboBox *m_tpgColorspace;
+	QComboBox *m_tpgXferFunc;
+	QComboBox *m_tpgYCbCrEnc;
+	QComboBox *m_tpgQuantRange;
 	bool m_useTpg;
 	QCheckBox *m_tpgLimRGBRange;
 
@@ -245,7 +263,9 @@ private:
 	int m_increment;
 	GeneralTab *m_genTab;
 	VbiTab *m_vbiTab;
+	QMenu *m_capMenu;
 	QAction *m_capStartAct;
+	QAction *m_capStepAct;
 	QAction *m_saveRawAct;
 	QAction *m_useGLAct;
 	QAction *m_audioBufferAct;
