@@ -26,6 +26,7 @@
 #include <string>
 #include <map>
 #include <set>
+#include <map>
 #include <linux/videodev2.h>
 
 #ifndef NO_LIBV4L2
@@ -49,6 +50,8 @@ struct test_query_ext_ctrl: v4l2_query_ext_ctrl {
 
 typedef std::map<__u32, struct test_query_ext_ctrl> qctrl_map;
 typedef std::set<__u32> pixfmt_set;
+typedef std::set<__u64> frmsizes_set;
+typedef std::map<__u32, unsigned> frmsizes_count_map;
 
 struct base_node;
 
@@ -61,6 +64,7 @@ struct base_node {
 	bool is_planar;
 	bool can_capture;
 	bool can_output;
+	bool can_scale;
 	const char *device;
 	struct node *node2;	/* second open filehandle */
 	bool has_outputs;
@@ -75,6 +79,9 @@ struct base_node {
 	unsigned std_controls;
 	unsigned priv_controls;
 	__u32 fbuf_caps;
+	pixfmt_set buftype_pixfmts[V4L2_BUF_TYPE_SDR_CAPTURE + 1];
+	frmsizes_set frmsizes;
+	frmsizes_count_map frmsizes_count;
 	__u32 valid_buftypes;
 	__u32 valid_buftype;
 	__u32 valid_memorytype;
@@ -100,6 +107,19 @@ struct node : public base_node, public cv4l_fd {
  			printf("\t\twarn: %s(%d): " fmt, __FILE__, __LINE__, ##args);	\
 	} while (0)
 
+#define warn_once(fmt, args...)						\
+	do {								\
+		static bool show;					\
+									\
+		if (!show) {						\
+			show = true;					\
+			warnings++;					\
+			if (show_warnings)				\
+				printf("\t\twarn: %s(%d): " fmt,	\
+					__FILE__, __LINE__, ##args); 	\
+		}							\
+	} while (0)
+
 #define fail(fmt, args...) 						\
 ({ 									\
  	printf("\t\tfail: %s(%d): " fmt, __FILE__, __LINE__, ##args);	\
@@ -137,6 +157,8 @@ const char *ok(int res);
 int check_string(const char *s, size_t len);
 int check_ustring(const __u8 *s, int len);
 int check_0(const void *p, int len);
+int restoreFormat(struct node *node);
+std::string pixfmt2s(unsigned id);
 
 // Debug ioctl tests
 int testRegister(struct node *node);
@@ -179,6 +201,9 @@ int testGetFormats(struct node *node);
 int testTryFormats(struct node *node);
 int testSetFormats(struct node *node);
 int testSlicedVBICap(struct node *node);
+int testCropping(struct node *node);
+int testComposing(struct node *node);
+int testScaling(struct node *node);
 
 // Codec ioctl tests
 int testEncoder(struct node *node);
@@ -192,5 +217,10 @@ int testExpBuf(struct node *node);
 int testMmap(struct node *node, unsigned frame_count);
 int testUserPtr(struct node *node, unsigned frame_count);
 int testDmaBuf(struct node *expbuf_node, struct node *node, unsigned frame_count);
+void streamAllFormats(struct node *node);
+
+// Color tests
+int testColorsAllFormats(struct node *node, unsigned component,
+			 unsigned skip, unsigned perc);
 
 #endif
